'''
Created on 24 mei 2018

@author: ex03210
'''

#import files
import unittest

from nemonet.engines.graph import Graph, Vertex


#tests
class Test(unittest.TestCase):

    def setUp(self):
        self.tree = Graph()
        a = Vertex('A')
        b = Vertex('B')
        c = Vertex('C')
        d = Vertex('D')
        e = Vertex('E')
        f = Vertex('F')

        self.tree.addVertex(a)
        self.tree.addVertex(b)
        self.tree.addVertex(c)
        self.tree.addVertex(d)
        self.tree.addVertex(e)
        self.tree.addVertex(f)

        self.assertTrue(a.nbrOfEdges() == 0)
        self.assertTrue(b.nbrOfEdges() == 0)
        self.assertTrue(c.nbrOfEdges() == 0)
        self.assertTrue(d.nbrOfEdges() == 0)
        self.assertTrue(e.nbrOfEdges() == 0)
        self.assertTrue(f.nbrOfEdges() == 0)

        self.tree.addEdge( a, b )
        self.tree.addEdge( a, c )
        self.tree.addEdge( b, d )
        self.tree.addEdge( b, e )
        self.tree.addEdge( d, f )

        self.assertTrue(a.nbrOfEdges() == 2)
        self.assertTrue(b.nbrOfEdges() == 2)
        self.assertTrue(c.nbrOfEdges() == 0)
        self.assertTrue(d.nbrOfEdges() == 1)
        self.assertTrue(e.nbrOfEdges() == 0)
        self.assertTrue(f.nbrOfEdges() == 0)





    def test_leaf(self):
        l = self.tree.verticesWithNoEdges()
        self.assertTrue(len( l ) == 3 )
        names = [x.name for x in l]
        self.assertTrue( 'C' in names )
        self.assertTrue( 'E' in names )
        self.assertTrue( 'F' in names )

    def test_degree(self):
        # it is possible to figure out what the root node is,
        # it is the node (or vertex) with only outgoing edges
        v = self.tree.getVertex('A')
        self.assertTrue( self.tree.isStart(v) )
        self.assertFalse( self.tree.isEnd(v) )

        v = self.tree.getVertex('B')
        self.assertFalse( self.tree.isStart(v) )
        self.assertFalse( self.tree.isEnd(v) )

        vc = self.tree.getVertex('C')
        ve = self.tree.getVertex('E')
        vf = self.tree.getVertex('F')
        self.assertFalse(self.tree.isStart( vc ) )
        self.assertFalse(self.tree.isStart( ve ) )
        self.assertFalse(self.tree.isStart( vf ) )

        self.assertTrue(self.tree.isEnd( vc ) )
        self.assertTrue(self.tree.isEnd( ve ) )
        self.assertTrue(self.tree.isEnd( vf ) )

    def test_startEndPointPaths(self):
        #get a list of vertices and check if it is a start point
        #get a list of vertices and check if it is an end point
        #check if a path exist between al begin and end points
        startPoints = []
        endPoints = []
        nameList = self.tree.getAllNames()

        for name in nameList:
            v = self.tree.getVertex( name )
            if self.tree.isStart( v ):
                startPoints.append( v )
            if self.tree.isEnd( v ):
                endPoints.append( v )

        self.assertTrue( len( startPoints ) == 1 )
        self.assertTrue( len( endPoints ) == 3 )

        paths = []

        for start in startPoints:
            for end in endPoints:
                paths.append( self.tree.find_path(start.name, end.name ) )

        self.assertTrue( ['A', 'C'] in paths )
        self.assertTrue( ['A', 'B', 'E'] in paths )
        self.assertTrue( ['A', 'B', 'D', 'F'] in paths )


    def test_path(self):
        self.assertTrue( ['A', 'B', 'D', 'F'] == self.tree.find_path( 'A', 'F' ) )
        self.assertTrue( [ 'A', 'B', 'E'] == self.tree.find_path('A', 'E'))

    def tearDown(self):
        self.tree = None


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


# Created by Jan Rummens at 3/12/2020
from nemonet.engines.graph import Graph
from nemonet.engines.sequencer import Sequences
from nemonet.seleniumwebdriver.commands import Command
from nemonet.screencast.recording import ScreenRecorder
from selenium import webdriver
import json
import time

selenium_config = {
    "driver": {
        "headless": False,
        "browser": "chrome",
        "type": "selenium"
    },
    "recording": {
        "switch": False
    }
}

class TestDriver:

    def open(self):
        pass

    def close(self):
        pass

class ChromeTestDriver( TestDriver ):

    def __init__(self):
        self.driver = None

    def open(self):
        self.driver = webdriver.Chrome()
        self.driver.maximize_window()

    def close(self):
        self.driver.close()

    def get_native_driver(self):
        return self.driver


class Runner( object ):

    def __init__( self, runner_config=None):
        self.data = None
        self.driver = None

        if runner_config == None:
            self.data = selenium_config
        else:
            with open(runner_config, 'r') as fp:
                self.data = json.load(fp)

        assert list( self.data.keys() ) == ['driver', 'recording']
        assert list( self.data["driver"].keys() ) == ['headless', 'browser', 'type']
        assert list( self.data["recording"].keys()) == ['switch']

        if self.data["driver"]["type"] == "selenium":
            if self.data["driver"]["browser"] == "Chrome":
                self.driver = ChromeTestDriver()

    def turn_on_recording(self):
        if not self.is_recording:
            self.screenrecording.start()
            self.is_recording = True
            time.sleep(0.25)

    def turn_off_recording(self):
        if self.is_recording:
            self.screenrecording.stop()
            time.sleep(0.25)

    def execute_scenario(self, xml_files_name):
        if self.data["recording"]['switch']:
            self.is_recording = False
            self.screenrecording = ScreenRecorder()
            self.turn_on_recording()
        graph = Graph()
        graph.build(xml_files_name)
        seqences = Sequences(graph)
        commands = Command(self.driver.get_native_driver())
        commands.executeSequences(seqences, graph)
        self.driver.close()



# Created by Jan Rummens at 5/01/2021
import unittest
import json
from nemonet.runner.vision_runner import Runner

class RunnerConfigTestCase(unittest.TestCase):


    def setUp(self):
        json_config_dict = {}
        sub_config = {}
        sub_config["headless"] = False
        sub_config["browser"] = "chrome"
        sub_config["type"] = "selenium"
        json_config_dict["driver"] = sub_config

        sub_config = {}
        sub_config["switch"] = True
        json_config_dict["recording"] = sub_config

        self.filename = 'runner_config.json'

        with open( self.filename, 'w') as fp:
            json.dump(json_config_dict, fp, indent=4)
            fp.close()


    def test_read_format(self):
        with open('runner_config.json', 'r') as fp:
            data = json.load(fp)

        self.assertTrue( list( data.keys() ) == ['driver', 'recording'] )
        self.assertTrue( list( data["driver"].keys() ) == ['headless', 'browser', 'type'] )
        self.assertTrue( list( data["recording"].keys()) == ['switch'] )


    def test_configured_test_runner(self):
        runner = Runner()

if __name__ == '__main__':
    unittest.main()



# Created by Jan Rummens at 8/01/2021
import sys
from nemonet.runner.vision_runner import Runner
import typer

app = typer.Typer()

@app.command("scenario")
def start(scenario: str):
    try:
        runner = Runner()
        runner.execute_scenario(scenario)
    except ValueError:
        typer.echo(f"invalid commandline")
        raise typer.Exit(code=1)


def main():
    app()

